#
# Cookbook Name:: hipsnip-mongodb
# Attributes:: default
#
# Copyright 2013, HipSnip Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

################################################################################
# Download and install

default['mongodb']['download']['version'] = '2.4.10'
default['mongodb']['download']['checksum'] = 'fd5c88a45e5ebffb44a932be5b96910930b5891bf946ed3116a270fd8d71eb38' # SHA-256
# don't change this, unless you're running your own download server
default['mongodb']['download']['host'] = 'fastdl.mongodb.org'
default['mongodb']['download']['subfolder'] = 'linux/'


################################################################################
# Generic configuration (will be the same for all instances on this node)

default['mongodb']['user'] = 'mongodb'
default['mongodb']['group_id'] = 3500
default['mongodb']['group'] = 'mongodb'

# Paths
default['mongodb']['log_dir'] = '/var/log/mongo'
# we'll create a new folder in these paths for each instance
default['mongodb']['data_dir'] = '/var/lib/mongo_data'
default['mongodb']['journal_dir'] = '/var/lib/mongo_journal'

# Journaling
default['mongodb']['journal'] = true
default['mongodb']['journal_commit_interval'] = 100 # ms

# Logging
default['mongodb']['syslog'] = true # Having this ON means all log entries will go to syslog rather than to the log file (version >2.1)
default['mongodb']['log_append'] = true
default['mongodb']['log_cpu'] = false
default['mongodb']['log_verbose'] = false
default['mongodb']['log_quiet'] = true

# Authentication
default['mongodb']['auth_enabled'] = false # Below attributes only apply if auth is enabled
default['mongodb']['auth_keyfile'] = '/etc/mongodb/keyfile' # Specify a different path if the keyfile will be put in place by a separate process
# people should change this! As per the documentation, it was generated by running: openssl rand -base64 741
default['mongodb']['auth_keyfile_data'] = "aDebbWARtiKLuhw/gd3R6vSFJDLW1PQZF5o16B2CZq0zwDWbOuPnALt08q/Ef8kG\nCTxVg0jGigUU/UyJ5jbRLWqdD0AALNExfHdtM+OR8hQwv/8uFNbVhJFrV4SdWXkL\nhVs8zkiYVg+xbU8EX/l1rzmAtjygZRITLh+edTcbWkctK5ZepRsFK4NDkoKr3w2H\nZs3XQliy1UmL3JI58/S+/Z35MfNXNv0dr+oWahNV3RlGpQXWtnaT8/q+hnKWsTD4\nIUwkErr/oEYP2hHfp6sXc/mQ1rrSG6dYmlKzXE4jICRuOi5S+8v/QE+5F2zkmaW8\nsvOBYahvCBOXR0qYWq8cA3kFbXkXwrfk1QPvqTsW1mr5pz4zYmoctdODxph0CE2D\nqqMoJWdvSSRzMdTJD0Xguxy5YmQ3QRig2nb+qOZt7TYob0RgpblZfj72kJhMHj0S\nicGXZXNZgC3Zll7f8QqdEhmGviQaf9CXqSflbH1ev1TxySkOVabsoOz5R3YlOhSO\nUtLBjQi7AD7H13Iu1fjTWSnWrstgNEMdKqWBaoOrnde28n3HR+mfxyoiYLPGjv4E\nKDfpnUxL/1qyYMiym7heyIFLdIkXJC1hBzOzkh1kK6ifVt77eWZJ1OsQSzkmK+G8\nCS+aEuJdH4btbOUexge0sm0kXhcdHTrmPimY4nGgk87jUx8lf7c74iYhn2yLa5R0\n2BRuwDqdzVSd2/BU5LStm9r30iXJRgGAhBR73g/QcLrMtXnd4eoxByIH5u4wFvau\ndHJoQARQNokYxekQn8lLFFF7ETsrzhent5Iyk9KdZMPtVIbOOyb9ygDH/Dnokquo\nY6hcTp7Bu51JAX3f6e/BuH8lGDZsXvYAV92ZMQOb5tTssZZY6ySUFmGbKU2Ap6cx\nO5tTjAUOhHJsY4NWXqbrcx+WTSd7Nk38l1YlTs9PwPYxF1UrG0pYYCRgRxgCfsGK\nAUdJqJDqe7WuW0lcpdelHfShSXNj"
# Username and pass for default admin user
default['mongodb']['admin_user']['name'] = 'admin'
default['mongodb']['admin_user']['password'] = 'l1zKohO94Vf8' # people should still change this!
# Don't touch this, unless you know what you're doing! Doing so may break functionality in this cookbook.
default['mongodb']['admin_user']['roles'] = [ "readWriteAnyDatabase", "userAdminAnyDatabase", "dbAdminAnyDatabase", "clusterAdmin" ]

# Admin & Monitoring
default['mongodb']['http_enabled'] = false
default['mongodb']['rest_enabled'] = false
default['mongodb']['scripting_enabled'] = true
default['mongodb']['tablescan_enabled'] = true
default['mongodb']['prealloc_enabled'] = true

# Replication
default['mongodb']['oplog'] = 100 # MB

# misc
default['mongodb']['small_files'] = false
default['mongodb']['open_file_limit'] = 64000 # 10gen recommendation

default['mongodb']['set_tcp_keepalive_time'] = false
default['mongodb']['tcp_keepalive_time'] = 300 # 10gen recommendation


####################################################
# Gems

default['mongodb']['gem_version']['mongo'] = '1.10.0'
default['mongodb']['gem_version']['bson_ext'] = '1.10.0'


####################################################
# Settings for node health check provider

default['mongodb']['node_check']['retries'] = 3
default['mongodb']['node_check']['timeout'] = 10 # seconds
# @note - timeout is exponential - the actual timeout is retry*timeout


